/**************************************************************************
* foutstd.cpp                                                             *
*                                                                         *
* Interface:  foutstd.h                                                   *
* Implements: TStdFileOutput()                                            *
*                                                                         *
* See also output.cpp for additional comments of common methods.          *
**************************************************************************/

#include <stdio.h>
#include <string.h>
#include "foutstd.h"

/**************************************************************************
* fileerror TStdFileOutput::Open(bool append)                             *
*                                                                         *
* Opens the current file for writing (append mode if append is true)      *
* and resets the counter.                                                 *
**************************************************************************/
fileerror TStdFileOutput::Open(BOOL append)
{
	if(FFile)									// Is the file already open?
		return SetResult(ALREADYOPEN);
	if(!FFilename)								// Is the filename set?
		return SetResult(NOFILENAME);
	if(append ? ((FFile = fopen(FFilename,"a+b")) == NULL) :
		((FFile = fopen(FFilename,"w+b")) == NULL))
	{
		FFile = NULL;							// File could not be opened
		return SetResult(OPENERROR);
	}
	ResetCounter();							// Reset the counter
	return SetResult(OK);
}

/**************************************************************************
* fileerror TStdFileOutput::Close()                                       *
*                                                                         *
* Closes the current file.                                                *
**************************************************************************/
fileerror TStdFileOutput::Close()
{
	if(!FFile)									// Is the file open?
		return SetResult(NOTOPEN);
	if(fclose(FFile))
	{
		FFile = NULL;							// Error closing file
		return SetResult(CLOSEERROR);
	}
	FFile = NULL;
	return SetResult(OK);
}

/**************************************************************************
* fileerror TStdFileOutput::Write(char *buf, unsigned count)              *
*                                                                         *
* Writes count bytes from buf to the file and updates FLastWritten.       *
* Also updates the counter if FCounterActive is true.                     *
**************************************************************************/
fileerror TStdFileOutput::Write(char *buf, unsigned count)
{
	if(FResult)					// Don't do anything if an error already has occured
		return GetResult();
	if(!FFile)					// Is the file open?
		return SetResult(NOTOPEN);
	char *temp = new char[count];
	strcpy(temp,buf);
	size_t written = fwrite(temp,count,1,FFile);		// Perform the writing
	delete temp;
	if(written != 1)
		return SetResult(WRITEERROR);						// File write error
	else
	{
		if(FCounterActive)
		{
			char *pos = strrchr(buf,10);
			if(pos)				// Is there a physical line break in the write buffer?
				FCounter += strlen(pos) - 1;	// Yes, increase the counter with the
			else										// string length since the line break
				FCounter += strlen(buf);		// No, increase the counter with the
		}												// full buffer length
		FLastWritten = buf[strlen(buf)-1];	// Set the last written character
		return SetResult(OK);
	}
}

/**************************************************************************
* char *TStdFileOutput::SetFilename(char *NewName)                        *
*                                                                         *
* Sets the filename and returns the current. Should only be used          *
* before the file has been opened.                                        *
**************************************************************************/
char *TStdFileOutput::SetFilename(char *NewName)
{
	char *temp = "\0";
	if(FFilename)
	{
		strcpy(temp,FFilename);
		delete FFilename;
	}
	else
		temp = "";
	FFilename = new char[strlen(NewName)+1];
	strcpy(FFilename,NewName);
	return temp;
}

/**************************************************************************
* char *TStdFileOutput::GetFilename()                                     *
*                                                                         *
* Returns the current filename.                                           *
**************************************************************************/
char *TStdFileOutput::GetFilename()
{
	return FFilename;
}

/**************************************************************************
* TStdFileOutput::TStdFileOutput()                                        *
*                                                                         *
* Standard constructor, simply resets some variables.                     *
**************************************************************************/
TStdFileOutput::TStdFileOutput()
{
	FFilename = NULL;
	FFile = NULL;
	FCounter = 0;
	FCounterActive = FALSE;
}

/**************************************************************************
* TStdFileOutput::TStdFileOutput(char *filename)                          *
*                                                                         *
* Constructor, sets the filename in addition to resetting variables       *
**************************************************************************/
TStdFileOutput::TStdFileOutput(char *filename)
{
	FFilename = new char[strlen(filename)+1];
	strcpy(FFilename,filename);
	FFile = NULL;
	FCounter = 0;
	FCounterActive = FALSE;
}

/**************************************************************************
* TStdFileOutput::~TStdFileOutput()                                       *
*                                                                         *
* Destructor, closes the file if that isn't done and deallocates memory.  *
**************************************************************************/
TStdFileOutput::~TStdFileOutput()
{
	Close();
	if(FFilename)
		delete FFilename;
}
