/**************************************************************************
* fnames.cpp                                                              *
*                                                                         *
* Interface:  fnames.h                                                    *
* Implements: extractfilename()                                           *
*             extractfilepath()                                           *
*             getuniquefilename()                                         *
*             addtrailingbackslash()                                      *
*             fileexists()                                                *
**************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dir.h>
#include "fnames.h"
#include "basicdef.h"

/**************************************************************************
* void extractfilename(char *dest, char *src)                             *
*                                                                         *
* Extracts the file name with extension from a fully qualified path       *
* and filename.                                                           *
**************************************************************************/
void extractfilename(char *dest, char *src)
{
	char *temp = "\0";
	strcpy(temp,src);
	int index = strscanlast(temp,'\\');
	temp += /*&temp[*/index+1/*]*/;
	strcpy(dest,temp);
}


/**************************************************************************
* void extractfilepath(char *dest, char *src, bool trailingslash = false) *
*                                                                         *
* Extracts the path from a fully qualified path and filename.             *
* The last parameter tells whether a trailing backslash will be added     *
* to all paths, if it's false a backslash will only be added when the     *
* path is the root.                                                       *
**************************************************************************/
void extractfilepath(char *dest, char *src, BOOL trailingslash)
{
	char temp[MAXPATH] = "\0";
	strcpy(temp,src);
	int index = strscanlast(temp,'\\');
	if(index != -1)
	{
		if(((index > 3) && trailingslash) || (index <= 3))
			temp[index+1] = '\0';
		else
			temp[index] = '\0';
	}
	strcpy(dest,temp);
}

/**************************************************************************
* void buildfilename(char *dest, char *path, char prefix[5], int seq,     *
*    char *ext)                                                           *
*                                                                         *
* Builds a complete path from a path, a 4 character prefix, a number and  *
* an extension. The result is placed in dest.                             *
**************************************************************************/
void buildfilename(char *dest, char *path, char prefix[5], int seq, char *ext)
{
	strcpy(dest,path);
	addtrailingbackslash(dest);
	strcat(dest,prefix);
	char seqstr[5] = "\0\0\0\0\0";
	itoa(seq,seqstr,16);				// Convert to string using hex base
	char *temp = new char[5];
	strcpy(temp,"0000\0");			// Make sure 4 characters is used even when the
	strcat(temp,seqstr);				// number itself occupies less, pad with zeros
	while(strlen(temp) > 4)			// Only allow 4 characters for seq. number
		temp++;
	strcat(dest,temp);
	delete temp;
	strcat(dest,".");
	strcat(dest,ext);
}

/**************************************************************************
* void getuniquefilename(char *dest, char *path, char *prefix, char *ext) *
*                                                                         *
* Searches for the first unique filename in the directory <path>. The     *
* resulting filename will be of the form <path>\<prefix>nnnn.<ext> where  *
* nnnn is a sequential number (the first available).                      *
**************************************************************************/
void getuniquefilename(char *dest, char *path, char *prefix, char *ext)
{
	int i = 0;
	char *fullname = new char[MAXPATH];
	do
	{
		buildfilename(fullname,path,prefix,i,ext);
		i++;
	} while(fileexists(fullname));
	buildfilename(dest,path,prefix,i,ext);
	delete fullname;
}

/**************************************************************************
* void addtrailingbackslash(char *s)                                      *
*                                                                         *
* Ensures that the path specified in s has got a trailing backslash so    *
* that a filename easily can be concatenated.                             *
**************************************************************************/
void addtrailingbackslash(char *s)
{
	if(s[strlen(s)-1] != '\\')		// Is the last character a backslash
		strcat(s,"\\");				// It's not, so add one
}

/**************************************************************************
* bool fileexists(char *filename)                                         *
*                                                                         *
* Tests for existance of the file specified in filename.                  *
**************************************************************************/
BOOL fileexists(char *filename)
{
	FILE *f;
	if((f = fopen(filename,"r")) == NULL)
		return FALSE;
	else
	{
		fclose(f);
		return TRUE;
	}
}
