/**************************************************************************
* htmldoc.cpp                                                             *
*                                                                         *
* Interface:  htmldoc.h                                                   *
* Implements: TDCHtmlDocument                                             *
*                                                                         *
* See also document.cpp for additional comments of common methods.        *
**************************************************************************/

#include <stdlib.h>
#include <time.h>
#include <string.h>
#include "dclasses.h"
#include "htmldoc.h"

/**************************************************************************
* void TDCHtmlDocument::Translate(char *dest, char *src)                  *
*                                                                         *
* Replaces some extended characters with HTML entities to ensure platform *
* compatibility. A tab character is replaced by four non-breaking spaces. *
**************************************************************************/
void TDCHtmlDocument::Translate(char *dest, char *src)
{
	char *res = new char[BUF_SIZE];
	strcpy(res,"\0");
	for(int i=0;i<strlen(src);i++)
	{
		switch((unsigned char)src[i])
		{
			case '': 	strcat(res,"&aring;"); 		break;
			case '': 	strcat(res,"&auml;");		break;
			case '': 	strcat(res,"&ouml;");		break;
			case '': 	strcat(res,"&oslash;");		break;
			case '': 	strcat(res,"&uuml;");		break;
			case '': 	strcat(res,"&Aring;");		break;
			case '': 	strcat(res,"&Auml;");		break;
			case '': 	strcat(res,"&Ouml;");		break;
			case '': 	strcat(res,"&Oslash;");		break;
			case '': 	strcat(res,"&Uuml;");		break;
			case '<': 	strcat(res,"&lt;");			break;
			case '>': 	strcat(res,"&gt;");			break;
			case '\"':	strcat(res,"&quot;");		break;
			case '': 	strcat(res,"&szlig;");		break;
			case '&': 	strcat(res,"&amp;");			break;
			case '\t': 	strcat(res,"&nbsp;&nbsp;&nbsp;&nbsp;");		break;
			default:  	strcatchr(res,src[i]);
		}
	}
	strcpy(dest,res);
	delete res;
}

/**************************************************************************
* fileerror TDCHtmlDocument::EndListItemTags()                            *
*                                                                         *
* Closes a list item tag, called when the list item is broken up, i.e.    *
* by a paragraph break or a new list or list item.                        *
**************************************************************************/
fileerror TDCHtmlDocument::EndListItemTags()
{
	if(FListItemEndTag)
	{
		PutPhysical("</LI>");
		FListItemEndTag = FALSE;
	}
	return FOut->GetResult();
}

/**************************************************************************
* fileerror TDCHtmlDocument::EndParagraphTags()                           *
*                                                                         *
* Closes a paragraph break tag, called when new paragraphs are inserted   *
* and when the document is finished with a call to PutFooters().          *
**************************************************************************/
fileerror TDCHtmlDocument::EndParagraphTags()
{
	if(FParagraphEndTag)
	{
		PutPhysical("</P>");
		FParagraphEndTag = FALSE;
	}
	return FOut->GetResult();
}

BOOL TDCHtmlDocument::ActOnStyleSheet(int OldSS)
{
	EndListItemTags();
	switch (OldSS)
	{
		case SS_HEADING1:
			PutPhysical("\r\n</H1>\r\n");
			FOut->ResetCounter();
			break;
		case SS_HEADING2:
			PutPhysical("\r\n</H2>\r\n");
			FOut->ResetCounter();
			break;
		case SS_HEADING3:
			PutPhysical("\r\n</H3>\r\n");
			FOut->ResetCounter();
			break;
	}
	switch (FStyleSheet)
	{
		case SS_HEADING1:
			PutPhysical("\r\n<H1>\r\n");
			FOut->ResetCounter();
			break;
		case SS_HEADING2:
			PutPhysical("\r\n<H2>\r\n");
			FOut->ResetCounter();
			break;
		case SS_HEADING3:
			PutPhysical("\r\n<H3>\r\n");
			FOut->ResetCounter();
			break;
	}
	return TRUE;
}

BOOL TDCHtmlDocument::SetBold(BOOL Toggle)
{
	BOOL temp;
	temp = FBold;
	FBold = Toggle;
	if(FBold)
		PutPhysical("<B>");
	else
		PutPhysical("</B>");
	return temp;
}

BOOL TDCHtmlDocument::SetItalic(BOOL Toggle)
{
	BOOL temp;
	temp = FItalic;
	FItalic = Toggle;
	if(FItalic)
		PutPhysical("<I>");
	else
		PutPhysical("</I>");
	return temp;
}

ulstyle TDCHtmlDocument::SetUnderline(ulstyle Style)
{
	ulstyle temp = FUnderline;
	FUnderline = Style;
	switch(FUnderline)
	{
		case NONE:			PutPhysical("</U>"); break;
		default:				PutPhysical("<U>"); break;
	}
	return temp;
}

BOOL TDCHtmlDocument::SetStrikethrough(BOOL Toggle)
{
	BOOL temp;
	temp = FStrikethrough;
	FStrikethrough = Toggle;
	return temp;
}

fileerror TDCHtmlDocument::BeginBullets()
{
	EndListItemTags();
	PhysicalLineBreak();
	PutPhysical("\r\n<UL>");
	return FOut->GetResult();
}

fileerror TDCHtmlDocument::EndBullets()
{
	EndListItemTags();
	PhysicalLineBreak();
	PutPhysical("</UL>\r\n");
	return FOut->GetResult();
}

fileerror TDCHtmlDocument::NewBullet()
{
	EndListItemTags();
	PhysicalLineBreak();
	PutPhysical("<LI>");
	FListItemEndTag = TRUE;
	return FOut->GetResult();
}

fileerror TDCHtmlDocument::BeginNumbering()
{
	EndListItemTags();
	PhysicalLineBreak();
	PutPhysical("<OL>\r\n");
	return FOut->GetResult();
}

fileerror TDCHtmlDocument::EndNumbering()
{
	EndListItemTags();
	PhysicalLineBreak();
	PutPhysical("</OL>\r\n");
	return FOut->GetResult();
}

fileerror TDCHtmlDocument::NewNumber()
{
	FNumberingCounts[FNumberingLevel]++;
	return NewBullet();
}

fileerror TDCHtmlDocument::BeginTable(char *TableCaption, captionalign ca)
{
	PerformDefaultBeginTableAction(TableCaption,ca);
	PutPhysical("\n<P>\n<TABLE BORDER=1>\n");
	if(FTableCaption)
	{
		if(FTableCaptionAlignment == ABOVE)
			PutPhysical("<CAPTION ALIGN=TOP>\n");
		else
			PutPhysical("<CAPTION ALIGN=BOTTOM>\n");
		PutTableCaption();
		PutPhysical("\n</CAPTION>\n");
	}
	PutPhysical("<TR>\n");
	FRowOpen = TRUE;
	return OK;
}

fileerror TDCHtmlDocument::EndTable()
{
	if(FCellOpen)
		PutPhysical("\n</TD>");
	if(FRowOpen)
		PutPhysical("\n</TR>");
	PutPhysical("\n</TABLE>\n</P>\n\n");
	FCellOpen = FALSE;
	PerformDefaultEndTableAction();
	return OK;
}

fileerror TDCHtmlDocument::AddCell(int Width, int BorderStyle)
{
	FCurrentCol++;
	if(FCellOpen)
		PutPhysical("\n</TD>");
	FCellOpen = TRUE;
	FTableCellWidth = Width;
	if(!FTableCellWidth)
		PutPhysical("\n<TD>\n");
	else
	{
		PutPhysical("\n<TD WIDTH=");
		PutInt(FTableCellWidth*10);
		PutPhysical(">\n");
	}
	return FOut->GetResult();
}

fileerror TDCHtmlDocument::AddRow()
{
	if(FRowOpen)
		PutPhysical("\n</TR>");
	PutPhysical("\n<TR>\n");
	FRowOpen = TRUE;
	FCurrentRow++;
	FCurrentCol = -1;
	return FOut->GetResult();
}

fileerror TDCHtmlDocument::PutText(char *text)
{
	char *temp1 = new char[BUF_SIZE];
	char *temp2 = new char[BUF_SIZE];
	Translate(temp1,text);
	BreakLine(temp2,temp1);
	*FCellData[FCurrentCol][FCurrentRow] = atol(temp2);
	FOut->Write(temp2,strlen(temp2));
	delete temp1;
	delete temp2;
	return FOut->GetResult();
}

fileerror TDCHtmlDocument::PutHeaders()
{
	PutPhysical("<HTML>\r\n<HEAD>\r\n<TITLE>");
	PutText(FTitle);
	PutPhysical("</TITLE>\r\n\r\n<META NAME=GENERATOR CONTENT=\"");
	char temp[30] = "\0";
	strcpy(temp,dcGetVersion());
	PutText(temp);
	PutPhysical("\">\r\n");
	if(FAuthor)
	{
		time_t *lt = new time_t;
		time(lt);
		tm *t = localtime(lt);
		PutPhysical("<META NAME=COPYRIGHT CONTENT=\"Copyright (C) ");
		PutPhysical(1900 + t->tm_year);
		PutPhysical(" ");
		PutText(FAuthor);
		PutPhysical("\">\r\n");
		delete lt;
		if(!FKeywords)
			PhysicalLineBreak();
	}
	if(FKeywords)
	{
		PutPhysical("<META NAME=KEYWORDS CONTENT=\"");
		PutText(FKeywords);
		PutPhysical("\">\r\n\r\n");
	}
	PutPhysical("</HEAD>\r\n<BODY>\r\n");
	FOut->ResetCounter();
	return FOut->GetResult();
}

fileerror TDCHtmlDocument::PutFooters()
{
	EndListItemTags();
	EndParagraphTags();
	PutPhysical("\r\n</BODY>\r\n</HTML>");
	FOut->ResetCounter();
	return FOut->GetResult();
}

fileerror TDCHtmlDocument::PutDivider()
{
	PutPhysical("\r\n<HR>\r\n");
	FOut->ResetCounter();
	return FOut->GetResult();
}

fileerror TDCHtmlDocument::PutLink(char *LinkText, char *LinkTo)
{
	PutText("<A HREF=\"");
	PutText(LinkTo);
	PutText("\">");
	PutText(LinkText);
	PutText("</A>");
	return FOut->GetResult();
}

fileerror TDCHtmlDocument::Tab()
{
	return PutPhysical("&nbsp;&nbsp;&nbsp;&nbsp;");
}

fileerror TDCHtmlDocument::ParagraphBreak()
{
	EndListItemTags();
	EndParagraphTags();
	PutPhysical("\r\n<P>\r\n");
	FOut->ResetCounter();
	FParagraphEndTag = TRUE;
	return FOut->GetResult();
}

fileerror TDCHtmlDocument::LineBreak()
{
	PutPhysical("<BR>\r\n");
	FOut->ResetCounter();
	return FOut->GetResult();
}

fileerror TDCHtmlDocument::NewTopic(char *TopicTitle, char *TopicName)
{
	PhysicalLineBreak(2);
	PutDivider();
	PhysicalLineBreak(2);
	PutPhysical("<A NAME=\"",TopicName,"\">\r\n");
	SetStyleSheet(SS_HEADING1);
	PutText(TopicTitle);
	SetStyleSheet(SS_NORMAL);
	return FOut->GetResult();
}

int TDCHtmlDocument::GetIdent()
{
	return DT_HTML;
}

TDCHtmlDocument::TDCHtmlDocument(char *Filename, TOutput *OutputObj) :
	TDCDocument(Filename, OutputObj)
{
	strcpy(FExtension,"html");
	FCharset = ISO;
	FParagraphEndTag = FALSE;
	FListItemEndTag = FALSE;
	FCellOpen = FALSE;
	FRowOpen = FALSE;
}

TDCHtmlDocument::~TDCHtmlDocument()
{

}