#ifndef __HTMLDOC_H

#include "document.h"
#include "output.h"

// Document type identification constant
const int DT_HTML    = 3;

class TDCHtmlDocument : public TDCDocument
{
	protected:
//*** Protected data members
		BOOL FParagraphEndTag;		// Is there an open <P> tag?
		BOOL FListItemEndTag;		// Is there an open <LI> tag?
		BOOL FCellOpen;				// Has a <TD> tag been issued?
		BOOL FRowOpen;					// Has a <TR> tag been issued?
//*** Protected member functions
		virtual void Translate(char *src, char *dest);
		fileerror EndListItemTags();
		fileerror EndParagraphTags();
		virtual BOOL ActOnStyleSheet(int OldSS);
	public:
//*** Set text attributes
		virtual BOOL SetBold(BOOL Toggle);
		virtual BOOL SetItalic(BOOL Toggle);
		virtual ulstyle SetUnderline(ulstyle Style);
		virtual BOOL SetStrikethrough(BOOL Toggle);
//*** Bullets and numbering functions
		int GetBulletLevel();
		int GetNumberingLevel();
		virtual fileerror BeginBullets();
		virtual fileerror EndBullets();
		virtual fileerror NewBullet();
		virtual fileerror BeginNumbering();
		virtual fileerror EndNumbering();
		virtual fileerror NewNumber();
//*** Table functions
		virtual fileerror BeginTable(char *TableCaption = "",
			captionalign ca = ABOVE);
		virtual fileerror EndTable();
		virtual fileerror AddCell(int Width = 0, int BorderStyle = BS_NONE);
		virtual fileerror AddRow();
//*** Write text to the document
		virtual fileerror PutText(char *Text);
		virtual fileerror PutHeaders();
		virtual fileerror PutFooters();
		virtual fileerror PutDivider();
		virtual fileerror PutLink(char *LinkText, char *LinkTo);
		virtual fileerror Tab();
		virtual fileerror ParagraphBreak();
		virtual fileerror LineBreak();
		virtual fileerror NewTopic(char *TopicTitle, char *TopicName);
//*** Misc methods
		virtual int GetIdent();
//*** Public constructors and destructors
		TDCHtmlDocument(char *Filename, TOutput *OutputObj);
		~TDCHtmlDocument();
};

#define __HTMLDOC_H

#endif