/**************************************************************************
* rtfdoc.cpp                                                              *
*                                                                         *
* Interface:  rtfdoc.h                                                    *
* Implements: TDCRtfDocument                                              *
*                                                                         *
* See also document.cpp for additional comments of common methods.        *
**************************************************************************/

#include <stdlib.h>
#include <time.h>
#include <string.h>
#include "dclasses.h"
#include "rtfdoc.h"

/**************************************************************************
* void TDCRtfDocument::Translate(char *dest, char *src)                   *
*                                                                         *
* Since RTF is an 7-bit format, character with ASCII value > 127 is       *
* encoded as \'<ASCII in hex>. Braces and tabs are also converted, the    *
* former as they are RTF control tokens.                                  *
**************************************************************************/
void TDCRtfDocument::Translate(char *dest, char *src)
{
	char *res = new char[BUF_SIZE];
	strcpy(res,"\0");
	for(int i=0;i<strlen(src);i++)
	{
		char *hexstr = new char[5];
		strcpy(hexstr,"\0");
		if((unsigned char)src[i] > 127)
		{
				strcat(res,"\\'");
				tohexstr(hexstr,(unsigned char)src[i],2);
				strcat(res,hexstr);
		}
		else
		{
			switch((unsigned char)src[i])
			{
				case '{':
					strcat(res,"\\{");
					break;
				case '}':
					strcat(res,"\\}");
					break;
				case '\t':
					strcat(res,"\\tab ");
					break;
				default: strcatchr(res,src[i]);
			}
		}
		delete hexstr;
	}
	strcpy(dest,res);
	delete res;
}

BOOL TDCRtfDocument::ActOnStyleSheet(int OldSS)
{
	if(FIsFirstLineOfDoc)
		FIsFirstLineOfDoc = FALSE;
	else
		ParagraphBreak();
	switch (FStyleSheet)
	{
		case SS_NORMAL:		PutPhysical("\\plain\\pard \\s0 ",
			FStyleSheetDefs[SS_NORMAL]); break;
		case SS_FIRSTLINE:	PutPhysical("\\plain\\pard \\s1 ",
			FStyleSheetDefs[SS_FIRSTLINE]); break;
		case SS_HEADING1:		PutPhysical("\\plain\\pard \\s2 ",
			FStyleSheetDefs[SS_HEADING2]); break;
		case SS_HEADING2:		PutPhysical("\\plain\\pard \\s3 ",
			FStyleSheetDefs[SS_HEADING2]); break;
		case SS_HEADING3:		PutPhysical("\\plain\\pard \\s4 ",
			FStyleSheetDefs[SS_HEADING3]); break;
		case SS_CAPTION:		PutPhysical("\\plain\\pard \\s5 ",
			FStyleSheetDefs[SS_CAPTION]); break;
	}
	return TRUE;
}

/**************************************************************************
* fileerror TDCRtfDocument::PutFontTable()                                *
*                                                                         *
* Writes the font table with all used fonts to the header. Called in      *
* PutHeaders(). Can be overridden to provide additional fonts.            *
**************************************************************************/
fileerror TDCRtfDocument::PutFontTable()
{
	PutPhysical("{\\fonttbl");
	PutPhysical("{\\f0\\froman\\fcharset0\\fprq2 Times New Roman{\\falt Times};}");
	PutPhysical("{\\f1\\fswiss\\fcharset0\\fprq2 Arial{\\falt Helvetica};}");
	PutPhysical("{\\f2\\ftech\\fcharset2\\fprq2 Symbol;}");
	PutPhysical("}");
	return FOut->GetResult();
}

/**************************************************************************
* fileerror TDCRtfDocument::PutColorTable()                               *
*                                                                         *
* Writes the color table with all possible text colors to the header.     *
* Called in PutHeaders(). Can be overridden to provide additional fonts.  *
**************************************************************************/
fileerror TDCRtfDocument::PutColorTable()
{
	PutPhysical("{\\colortbl;");
	PutPhysical("\\red0\\green0\\blue0;");
	PutPhysical("\\red0\\green0\\blue255;");
	PutPhysical("\\red0\\green255\\blue255;");
	PutPhysical("\\red0\\green255\\blue0;");
	PutPhysical("\\red255\\green0\\blue255;");
	PutPhysical("\\red0\\green0\\blue128;");
	PutPhysical("\\red0\\green128\\blue128;");
	PutPhysical("\\red0\\green128\\blue0;");
	PutPhysical("\\red128\\green0\\blue128;");
	PutPhysical("\\red128\\green0\\blue0;");
	PutPhysical("\\red128\\green128\\blue0;");
	PutPhysical("\\red128\\green128\\blue128;");
	PutPhysical("\\red192\\green192\\blue192;");
	PutPhysical("\\red255\\green0\\blue0;");
	PutPhysical("\\red255\\green255\\blue0;");
	PutPhysical("\\red255\\green255\\blue255;");
	PutPhysical("}\r\n");
	return FOut->GetResult();
}

/**************************************************************************
* fileerror TDCRtfDocument::PutStyleSheets()                              *
*                                                                         *
* Writes the stylesheet table with all stylesheet definitions to the      *
* header. The definitions are stored in FStyleSheetDefs since they're     *
* also needed when choosing stylesheets. Called in PutHeaders(). Can be   *
* overridden together with SetStyleSheetDefs() to to provide additional   *
* stylesheets.                                                            *
**************************************************************************/
fileerror TDCRtfDocument::PutStyleSheets()
{
	PutPhysical("{\\stylesheet");
	PutPhysical("{\\s0 ",FStyleSheetDefs[SS_NORMAL]," Normal;}");
	if(FLanguage == LANG_ENGLISH)
		PutPhysical("{\\s1\\sbasedon0 ",FStyleSheetDefs[SS_FIRSTLINE],
			" First line indent;}");
	PutPhysical("{\\s2 ",FStyleSheetDefs[SS_HEADING1]," heading 1;}");
	PutPhysical("{\\s3 ",FStyleSheetDefs[SS_HEADING2]," heading 2;}");
	PutPhysical("{\\s4 ",FStyleSheetDefs[SS_HEADING3]," heading 3;}");
	PutPhysical("{\\s5 ",FStyleSheetDefs[SS_CAPTION]," caption;}");
	PutPhysical("}");
	return FOut->GetResult();
}

/**************************************************************************
* fileerror TDCRtfDocument::PutDocumentInfo()                             *
*                                                                         *
* Writes a document information block with author, title, keywords etc.   *
* to the header. Can be overridden to provide additional stylesheets.     *
**************************************************************************/
fileerror TDCRtfDocument::PutDocumentInfo()
{
	PutPhysical("{\\info");
	if(FTitle)
		PutPhysical("{\\title ",FTitle,"}");
	if(FAuthor)
	{
		PutPhysical("{\\author ");
		PutText(FAuthor);
		PutPhysical("}");
		PutPhysical("{\\operator ");
		PutText(FAuthor);
		PutPhysical("}");
	}
	if(FKeywords)
		PutPhysical("{\\keywords ",FKeywords,"}");
	time_t *lt = new time_t;
	tm *t = new tm;
	time(lt);
	t = localtime(lt);
	delete lt;
// Put the modification time
	PutPhysical("{\\creatim\\yr");
	PutPhysical(1900 + t->tm_year);
	PutPhysical("\\mo");
	PutPhysical(t->tm_mon+1);
	PutPhysical("\\dy");
	PutPhysical(t->tm_mday);
	PutPhysical("\\hr");
	PutPhysical(t->tm_hour);
	PutPhysical("\\min");
	PutPhysical(t->tm_min);
	PutPhysical("}");
// Put the revision time
	PutPhysical("{\\revtime\\yr");
	PutPhysical(1900 + t->tm_year);
	PutPhysical("\\mo");
	PutPhysical(t->tm_mon+1);
	PutPhysical("\\dy");
	PutPhysical(t->tm_mday);
	PutPhysical("\\hr");
	PutPhysical(t->tm_hour);
	PutPhysical("\\min");
	PutPhysical(t->tm_min);
	PutPhysical("}}");
	delete t;
//	Put userprops
	PutPhysical("{\\*\\userprops{\\propname ");
	switch(FLanguage)
	{
		case LANG_ENGLISH: 	PutPhysical("Creator"); break;
		case LANG_SWEDISH: 	PutPhysical("Skapare"); break;
		case LANG_GERMAN:  	PutPhysical("Erschaffer"); break;
		case LANG_FRENCH:  	PutPhysical("Createur?"); break;
		default: 				PutPhysical("Creator"); break;
	}
	PutPhysical("}\\proptype30{\\staticval ");
	char temp[30] = "\0";
	strcpy(temp,dcGetVersion());
	PutText(temp);
	PutPhysical("}}");
	return FOut->GetResult();
}

/**************************************************************************
* fileerror TDCRtfDocument::PutDocumentFormatting()                       *
*                                                                         *
* Writes a document information block with document settings such as      *
* margins. Can be overridden to provide additional stylesheets.           *
**************************************************************************/
fileerror TDCRtfDocument::PutDocumentFormatting()
{
	PutPhysical("\\makebackup");
	return FOut->GetResult();
}

/**************************************************************************
* fileerror TDCRtfDocument::PutSectionFormatting()                        *
*                                                                         *
* Writes a document information block with section formatting properties. *
* Can be overridden to provide additional stylesheets.                    *
**************************************************************************/
fileerror TDCRtfDocument::PutSectionFormatting()
{
	PutPhysical("\\sectd");
	PutPhysical("{\\*\\pnseclvl1\\pndec\\pnstart1\\pnindent");
	PutPhysical(NUMBERINGINDENTFACTOR);
	PutPhysical("\\pnhang{\\pntxta .}}");
	PutPhysical("{\\*\\pnseclvl2\\pnucltr\\pnstart1\\pnindent");
	PutPhysical(NUMBERINGINDENTFACTOR);
	PutPhysical("\\pnhang{\\pntxta .}}");
	PutPhysical("{\\*\\pnseclvl3\\pnucrm\\pnstart1\\pnindent");
	PutPhysical(NUMBERINGINDENTFACTOR);
	PutPhysical("\\pnhang{\\pntxta .}}");
	PutPhysical("{\\*\\pnseclvl4\\pnlcltr\\pnstart1\\pnindent");
	PutPhysical(NUMBERINGINDENTFACTOR);
	PutPhysical("\\pnhang{\\pntxta .}}");
	PutPhysical("{\\*\\pnseclvl5\\pndec\\pnstart1\\pnindent");
	PutPhysical(NUMBERINGINDENTFACTOR);
	PutPhysical("\\pnhang{\\pntxta .}}");
	PutPhysical("{\\*\\pnseclvl6\\pndec\\pnstart1\\pnindent");
	PutPhysical(NUMBERINGINDENTFACTOR);
	PutPhysical("\\pnhang{\\pntxta .}}");
	PutPhysical("{\\*\\pnseclvl7\\pndec\\pnstart1\\pnindent");
	PutPhysical(NUMBERINGINDENTFACTOR);
	PutPhysical("\\pnhang{\\pntxta .}}");
	PutPhysical("{\\*\\pnseclvl8\\pndec\\pnstart1\\pnindent");
	PutPhysical(NUMBERINGINDENTFACTOR);
	PutPhysical("\\pnhang{\\pntxta .}}");
	PutPhysical("{\\*\\pnseclvl9\\pndec\\pnstart1\\pnindent");
	PutPhysical(NUMBERINGINDENTFACTOR);
	PutPhysical("\\pnhang{\\pntxta .}}");
	return FOut->GetResult();
}

/**************************************************************************
* void TDCRtfDocument::SetStyleSheetDefs()                                *
*                                                                         *
* Sets the FStyleSheetDefs array with the formatting properties the are   *
* special for this style sheet. Can be overridden to provide additional   *
* stylesheets.                                                            *
**************************************************************************/
void TDCRtfDocument::SetStyleSheetDefs()
{
	strcpy(FStyleSheetDefs[SS_NORMAL],"\\f0\\fs24 ");
	strcpy(FStyleSheetDefs[SS_FIRSTLINE],"\\fi283 ");
	strcpy(FStyleSheetDefs[SS_HEADING1],"\\hyphpar0\\keep\\sb240\\sa120\\f1\\fs36\\b ");
	strcpy(FStyleSheetDefs[SS_HEADING2],"\\hyphpar0\\keep\\sb240\\sa120\\f1\\fs28\\b ");
	strcpy(FStyleSheetDefs[SS_HEADING3],"\\hyphpar0\\keep\\sb240\\sa120\\f0\\fs26\\b ");
	strcpy(FStyleSheetDefs[SS_CAPTION],"\\sb120\\sa120\\nowidctlpar\\b\\f4 \\sbasedon0\\snext0 ");
}

BOOL TDCRtfDocument::SetBold(BOOL Toggle)
{
	BOOL temp;
	temp = FBold;
	FBold = Toggle;
	if(FBold)
		PutPhysical("\\b ");
	else
		PutPhysical("\\b0 ");
	return temp;
}

BOOL TDCRtfDocument::SetItalic(BOOL Toggle)
{
	BOOL temp;
	temp = FItalic;
	FItalic = Toggle;
	if(FItalic)
		PutPhysical("\\i ");
	else
		PutPhysical("\\i0 ");
	return temp;
}

ulstyle TDCRtfDocument::SetUnderline(ulstyle Style)
{
	ulstyle temp = FUnderline;
	FUnderline = Style;
	switch(FUnderline)
	{
		case NONE:			PutPhysical("\\ulnone"); break;
		case SINGLE:	 	PutPhysical("\\ul"); break;
		case DOUBLE:		PutPhysical("\\uldb"); break;
		case BYWORD:		PutPhysical("\\ulw"); break;
		default:				PutPhysical("\\ul"); break;
	}
	return temp;
}

BOOL TDCRtfDocument::SetStrikethrough(BOOL Toggle)
{
	BOOL temp;
	temp = FStrikethrough;
	FStrikethrough = Toggle;
	if(FStrikethrough)
		PutPhysical("\\strike ");
	else
		PutPhysical("\\strike0 ");
	return temp;
}

BOOL TDCRtfDocument::SetHidden(BOOL Toggle)
{
	BOOL temp;
	temp = FHidden;
	FHidden = Toggle;
	if(FHidden)
		PutPhysical("\\v ");
	else
		PutPhysical("\\v0 ");
	return temp;
}

fileerror TDCRtfDocument::BeginBullets()
{
	if(FBulletLevel < MAX_BULLET_NUMBERING_LEVEL)
		FBulletLevel++;
	FIsFirstBullet = TRUE;
	return OK;
}

fileerror TDCRtfDocument::EndBullets()
{
	if(FBulletLevel > 0)
		FBulletLevel--;
	FIsFirstBullet = TRUE;
	FIsFirstNumber = TRUE;
	return OK;
}

fileerror TDCRtfDocument::NewBullet()
{
	PutPhysical("\r\n\\par {\\pntext\\pard\\plain\\f2\\'b7\\tab}");
	if(FIsFirstBullet)
	{
		PutPhysical("\\pard \\fi-");
		PutPhysical(NUMBERINGINDENTFACTOR);
		PutPhysical("\\li");
		PutPhysical(FBulletLevel * NUMBERINGINDENTFACTOR);
		PutPhysical("\\nowidctlpar{\\*\\pn \\pnlvlblt");
		PutPhysical("\\pnf2\\pnstart1\\pnindent");
		PutPhysical(FBulletLevel * NUMBERINGINDENTFACTOR);
		PutPhysical("\\pnhang{\\pntxtb ");
		PutPhysical("\\'b7}}");
		FIsFirstBullet = FALSE;
	}
	return FOut->GetResult();
}

fileerror TDCRtfDocument::BeginNumbering()
{
	if(FNumberingLevel < MAX_BULLET_NUMBERING_LEVEL)
	{
		FNumberingLevel++;
		FNumberingCounts[FNumberingLevel] = 1;
	}
	FIsFirstNumber = TRUE;
	return OK;
}

fileerror TDCRtfDocument::EndNumbering()
{
	if(FNumberingLevel > 0)
		FNumberingLevel--;
	FIsFirstBullet = TRUE;
	FIsFirstNumber = TRUE;
	return OK;
}

fileerror TDCRtfDocument::NewNumber()
{
	PutPhysical("\r\n\\par {\\pntext\\pard\\plain ");
	PutPhysical(FNumberingCounts[FNumberingLevel]);
	PutPhysical(".\\tab}");
	if(FIsFirstNumber)
	{
		PutPhysical("\\pard \\fi-");
		PutPhysical(NUMBERINGINDENTFACTOR);
		PutPhysical("\\li");
		PutPhysical(FNumberingLevel * NUMBERINGINDENTFACTOR);
		PutPhysical("\\nowidctlpar{\\*\\pn ");
		PutPhysical("\\pnlvl");
		PutPhysical(FNumberingLevel);
		PutPhysical("\\pnstart1\\pnindent");
		PutPhysical(NUMBERINGINDENTFACTOR);
		PutPhysical("\\pnhang{\\pntxta .}}");
		FIsFirstNumber = FALSE;
	}
	FNumberingCounts[FNumberingLevel]++;
	return FOut->GetResult();
}

fileerror TDCRtfDocument::BeginTable(char *TableCaption, captionalign ca)
{
	PerformDefaultBeginTableAction(TableCaption,ca);
	FAccumulatedTableWidth = 0;
	if(FTableCaption && (FTableCaptionAlignment == ABOVE))
	{
		SetStyleSheet(SS_CAPTION);
		PutPhysical(" \\keepn ");
		switch(FLanguage)
		{
			case LANG_ENGLISH:	PutText("Table "); break;
			case LANG_SWEDISH:	PutText("Tabell "); break;
			case LANG_GERMAN:		PutText("Table "); break;
			case LANG_FRENCH:		PutText("Table "); break;
			default:					PutText("Table "); break;
		}
		PutPhysical("{\\field\\flddirty{\\*\\fldinst  SEQ ");
		switch(FLanguage)
		{
			case LANG_ENGLISH:	PutText("Table "); break;
			case LANG_SWEDISH:	PutText("Tabell "); break;
			case LANG_GERMAN:		PutText("Table "); break;
			case LANG_FRENCH:		PutText("Table "); break;
			default:					PutText("Table "); break;
		}
		PutPhysical("\\\\* ARABIC }{\\fldrslt {\\lang1024 1}}}: ");
		PutText(FTableCaption);
		SetStyleSheet(SS_NORMAL);
	}
	else
		PutPhysical("\r\n\\par ");
	PutPhysical("\\trowd \\trgaph");
	PutInt(RTFTABLECELLSPACING);
	PutPhysical("\\trleft-");
	PutInt(RTFTABLECELLSPACING);
	PutPhysical(" \\pard \\widctlpar\\intbl ");
	return FOut->GetResult();
}

fileerror TDCRtfDocument::EndTable()
{
	if(FTableCaption && (FTableCaptionAlignment == BELOW))
		PutPhysical(" \\keepn ");
	PutPhysical("\\cell \\pard \\widctlpar\\intbl \\row \\pard\\plain");
	if(FTableCaption && (FTableCaptionAlignment == BELOW))
	{
//*** Fool ActOnStyleSheetChange() into thinking this is the first line of the
//*** document, thereby a paragraph break isn't inserted.
		FIsFirstLineOfDoc = TRUE;
		SetStyleSheet(SS_CAPTION);
		switch(FLanguage)
		{
			case LANG_ENGLISH:	PutText("Table "); break;
			case LANG_SWEDISH:	PutText("Tabell "); break;
			case LANG_GERMAN:		PutText("Table "); break;
			case LANG_FRENCH:		PutText("Table "); break;
			default:					PutText("Table "); break;
		}
		PutPhysical("{\\field\\flddirty{\\*\\fldinst  SEQ ");
		switch(FLanguage)
		{
			case LANG_ENGLISH:	PutText("Table "); break;
			case LANG_SWEDISH:	PutText("Tabell "); break;
			case LANG_GERMAN:		PutText("Table "); break;
			case LANG_FRENCH:		PutText("Table "); break;
			default:					PutText("Table "); break;
		}
		PutPhysical("\\\\* ARABIC }{\\fldrslt {\\lang1024 1}}}: ");
		PutText(FTableCaption);
		FIsFirstLineOfDoc = TRUE;
		SetStyleSheet(SS_NORMAL);
	}
	else
		PutPhysical(" \\pard\\plain");
	PerformDefaultEndTableAction();
	PutPhysical("\r\n\\par ");
	return FOut->GetResult();
}

fileerror TDCRtfDocument::AddCell(int Width, int BorderStyle)
{
	FCurrentCol++;
	if(Width)
		FTableCellWidth = Width * 100;
	else
		FTableCellWidth = 2000;
	FAccumulatedTableWidth += FTableCellWidth + RTFTABLECELLSPACING;
	if(FCurrentCol)
		PutPhysical("\\cell ");
	PutPhysical("\\cellx");
	PutInt(FAccumulatedTableWidth);
	PutPhysical(" ");
	return FOut->GetResult();
}

fileerror TDCRtfDocument::AddRow()
{
	PutPhysical("\\cell \\pard \\widctlpar\\intbl \\row \\trowd \\trgaph");
	PutInt(RTFTABLECELLSPACING);
	PutPhysical("\\trleft-");
	PutInt(RTFTABLECELLSPACING);
	PutPhysical(" \\pard \\widctlpar\\intbl ");
	FCurrentRow++;
	FCurrentCol = -1;
	FAccumulatedTableWidth = 0;
	return FOut->GetResult();
}

fileerror TDCRtfDocument::PutText(char *text)
{
	char *temp1 = new char[BUF_SIZE];
	char *temp2 = new char[BUF_SIZE];
	Translate(temp1,text);
	BreakLine(temp2,temp1);
	*FCellData[FCurrentCol][FCurrentRow] = atol(temp2);
	FOut->Write(temp2,strlen(temp2));
	delete temp1;
	delete temp2;
	return FOut->GetResult();
}

fileerror TDCRtfDocument::PutHeaders()
{
	PutPhysical("{\\rtf1\\ansi \\deff4\\deflang");
	switch(FLanguage)
	{
		case LANG_ENGLISH: 	PutPhysical(0x0409); break;
		case LANG_SWEDISH: 	PutPhysical(0x041d); break;
		case LANG_GERMAN: 	PutPhysical(0x0407); break;
		case LANG_FRENCH: 	PutPhysical(0x040c); break;
		default: 				PutPhysical(0x0409); break;
	}
	PutFontTable();
	PutColorTable();
	PutStyleSheets();
	PutDocumentInfo();
	PutDocumentFormatting();
	PutSectionFormatting();
	return FOut->GetResult();
}

fileerror TDCRtfDocument::PutFooters()
{
	return PutPhysical("}");
}

fileerror TDCRtfDocument::ParagraphBreak()
{
	FIsFirstLineOfDoc = FALSE;
	if(FIsTable)
		return PutPhysical("\r\n\\par \\intbl");
	else
		return PutPhysical("\r\n\\par ");
}

fileerror TDCRtfDocument::Tab()
{
	return PutPhysical("\tab ");
}

fileerror TDCRtfDocument::PageBreak()
{
	return PutPhysical("\r\n\\page ");
}

fileerror TDCRtfDocument::LineBreak()
{
	return PutPhysical("\r\n\\line ");
}

fileerror TDCRtfDocument::NewTopic(char *TopicTitle, char *TopicName)
{
	SetStyleSheet(SS_HEADING1);
	PutText(TopicTitle);
	SetStyleSheet(SS_NORMAL);
	return FOut->GetResult();
}

int TDCRtfDocument::GetIdent()
{
	return DT_RTF;
}

TDCRtfDocument::TDCRtfDocument(char *Filename, TOutput *OutputObj) :
	TDCDocument(Filename, OutputObj)
{
	strcpy(FExtension,"RTF");
	FCharset = PCDOS;
	FIsFirstBullet = TRUE;
	FIsFirstNumber = TRUE;
	SetStyleSheetDefs();
}

TDCRtfDocument::~TDCRtfDocument()
{

}