#ifndef __RTFDOC_H

#include "document.h"

// Document type identification constant
const int DT_RTF		= 2;

const int NUMBERINGINDENTFACTOR 	= 283;
const int MAXSTYLESHEETDEFS     	= 20;
const int MAXSTYLESHEETDEFLENGTH	= 255;
const int RTFTABLECELLSPACING		= 70;

class TDCRtfDocument : public TDCDocument
{
	protected:
//*** Protected data members
		BOOL FIsFirstBullet;
		BOOL FIsFirstNumber;
		int FAccumulatedTableWidth;
		char FStyleSheetDefs[MAXSTYLESHEETDEFS-1][MAXSTYLESHEETDEFLENGTH];
//*** Protected members functions
		virtual void Translate(char *dest, char *src);
		virtual BOOL ActOnStyleSheet(int OldSS);
		virtual fileerror PutFontTable();
		virtual fileerror PutColorTable();
		virtual fileerror PutStyleSheets();
		virtual fileerror PutDocumentInfo();
		virtual fileerror PutDocumentFormatting();
		virtual fileerror PutSectionFormatting();
		virtual void SetStyleSheetDefs();
	public:
//*** Set text attributes
		virtual BOOL SetBold(BOOL Toggle);
		virtual BOOL SetItalic(BOOL Toggle);
		virtual ulstyle SetUnderline(ulstyle Style);
		virtual BOOL SetStrikethrough(BOOL Toggle);
		virtual BOOL SetHidden(BOOL Toggle);
//*** Bullet and numbering functions
		virtual fileerror BeginBullets();
		virtual fileerror EndBullets();
		virtual fileerror NewBullet();
		virtual fileerror BeginNumbering();
		virtual fileerror EndNumbering();
		virtual fileerror NewNumber();
//*** Table functions
		virtual fileerror BeginTable(char *TableCaption = "",
			captionalign ca = ABOVE);
		virtual fileerror EndTable();
		virtual fileerror AddCell(int Width = 0, int BorderStyle = BS_NONE);
		virtual fileerror AddRow();
//*** Write text to the document
		virtual fileerror PutText(char *Text);
		virtual fileerror PutHeaders();
		virtual fileerror PutFooters();
		virtual fileerror ParagraphBreak();
		virtual fileerror Tab();
		virtual fileerror PageBreak();
		virtual fileerror LineBreak();
		virtual fileerror NewTopic(char *TopicTitle, char *TopicName);
//*** Misc methods
		virtual int GetIdent();
//*** Public constructors and destructors
		TDCRtfDocument(char *Filename, TOutput *OutputObj);
		~TDCRtfDocument();
};

#define __RTFDOC_H

#endif