#ifndef __TEXTDOC_H

#include "document.h"
#include "output.h"

// Document type identification constant
const int DT_TEXT		= 1;

class TDCTextDocument : public TDCDocument
{
	protected:
		int FCellLength;				// Total length of the current cell contents
		virtual void BreakLine(char *src, char *dest);
		virtual fileerror PadCellToRequestedLength();
		virtual BOOL ActOnStyleSheet(int OldSS);
	public:
//*** Bullets and numbering functions
		int GetBulletLevel();
		int GetNumberingLevel();
		virtual fileerror BeginBullets();
		virtual fileerror EndBullets();
		virtual fileerror NewBullet();
		virtual fileerror BeginNumbering();
		virtual fileerror EndNumbering();
		virtual fileerror NewNumber();
//*** Table functions
		virtual fileerror BeginTable(char *TableCaption = "",
			captionalign ca = ABOVE);
		virtual fileerror EndTable();
		virtual fileerror AddCell(int Width = 0, int BorderStyle = BS_NONE);
		virtual fileerror AddRow();
//*** Write text to the document
		virtual fileerror PutText(char *Text);
		virtual fileerror PutDivider();
		virtual fileerror PutLink(char *LinkText, char *LinkTo);
		virtual fileerror Tab();
		virtual fileerror ParagraphBreak();
		virtual fileerror LineBreak();
		virtual fileerror PageBreak();
		virtual fileerror NewTopic(char *TopicTitle, char *TopicName);
//*** Misc methods
		virtual int GetIdent();
//*** Public constructors and destructors
		TDCTextDocument(char *Filename, TOutput *OutputObj);
		~TDCTextDocument();
};

#define __TEXTDOC_H

#endif